#ifndef STATUS_VIEW_H
#define STATUS_VIEW_H


class CStatusView;

class CStatusItem {
protected:
	CStatusView *m_pStatus;
	int m_ID;
	int m_DefaultWidth;
	int m_Width;
	int m_MinWidth;
	bool m_fVisible;
public:
	CStatusItem(int ID,int DefaultWidth);
	virtual ~CStatusItem() {}
	void SetStatusView(CStatusView *pStatus);
	int GetIndex() const;
	bool GetRect(RECT *pRect) const;
	int GetID() const { return m_ID; }
	int GetDefaultWidth() const { return m_DefaultWidth; }
	int GetWidth() const { return m_Width; }
	bool SetWidth(int Width);
	int GetMinWidth() const { return m_MinWidth; }
	void SetVisible(bool fVisible);
	bool GetVisible() const { return m_fVisible; }
	virtual LPCTSTR GetName() const=0;
	virtual void Draw(HDC hdc,const RECT *pRect)=0;
	virtual void DrawPreview(HDC hdc,const RECT *pRect) { Draw(hdc,pRect); }
	virtual void OnLButtonDown(int x,int y) {}
	virtual void OnRButtonDown(int x,int y) { OnLButtonDown(x,y); }
	virtual void OnMouseMove(int x,int y) {}
	virtual void OnVisibleChange(bool fVisible) {}
};

class CStatusView {
	static HINSTANCE m_hinst;
	HWND m_hwnd;
	HFONT m_hfontStatus;
	COLORREF m_crBackColor;
	COLORREF m_crTextColor;
	COLORREF m_crHighlightBackColor;
	COLORREF m_crHighlightTextColor;
	enum { MAX_STATUS_ITEMS=10 };
	CStatusItem *m_pItemList[MAX_STATUS_ITEMS];
	int m_NumItems;
	bool m_fSingleMode;
	LPTSTR m_pszSingleText;
	int m_HotItem;
	bool m_fTrackMouseEvent;
	static CStatusView *GetStatusView(HWND hwnd);
	static LRESULT CALLBACK WndProc(HWND hwnd,UINT uMsg,
												WPARAM wParam,LPARAM lParam);
public:
	static bool Initialize(HINSTANCE hinst);
	CStatusView();
	~CStatusView();
	bool Create(HWND hwndParent,int ID,DWORD Style,DWORD ExStyle);
	int NumItems() const { return m_NumItems; }
	const CStatusItem *GetItem(int Index) const;
	CStatusItem *GetItem(int Index);
	const CStatusItem *GetItemByID(int ID) const;
	CStatusItem *GetItemByID(int ID);
	bool AddItem(CStatusItem *pItem);
	int IDToIndex(int ID) const;
	int IndexToID(int Index) const;
	void UpdateItem(int ID);
	bool GetItemRect(int ID,RECT *pRect) const;
	void GetRect(RECT *pRect) const;
	int Height() const;
	int GetItemHeight() const;
	void Move(int x,int y,int Width,int Height);
	void SetVisible(bool fVisible);
	bool GetVisible() const;
	HWND GetHandle() const { return m_hwnd; }
	void SetSingleText(LPCTSTR pszText);
	void SetColor(COLORREF crBack,COLORREF crText,
							COLORREF crHighlightBack,COLORREF crHighlightText);
	bool SetFont(HFONT hfont);
	int GetCurItem() const;
	bool SetItemOrder(const int *pOrderList);
	bool DrawItemPreview(CStatusItem *pItem,HDC hdc,const RECT *pRect,bool fHighlight=false) const;
};


#endif
